@@ -25,7 +25,11 @@
 
 #include <glib/gi18n.h>
 
+#include "ptyxis-add-button-list-item.h"
+#include "ptyxis-add-button-list-model.h"
 #include "ptyxis-application.h"
+#include "ptyxis-custom-link-editor.h"
+#include "ptyxis-custom-link-row.h"
 #include "ptyxis-palette-preview.h"
 #include "ptyxis-preferences-list-item.h"
 #include "ptyxis-preferences-window.h"
@@ -34,6 +38,7 @@
 #include "ptyxis-shortcut-row.h"
 #include "ptyxis-util.h"
 
+
 /* This will not transition to AdwDialog until there is a way for
  * toplevel windows _with_ transient-for set to maintain window
  * group ordering.
@@ -49,10 +54,8 @@ struct _PtyxisPreferencesWindow
   GtkFilterListModel   *filter_palettes;
   guint                 filter_show_more : 1;
 
-  GtkButton            *add_profile_button;
   AdwSwitchRow         *audible_bell;
   AdwComboRow          *backspace_binding;
-  AdwSwitchRow         *bold_is_bright;
   AdwSpinRow           *cell_height_scale;
   AdwComboRow          *cjk_ambiguous_width;
   GListModel           *cjk_ambiguous_widths;
@@ -130,6 +133,7 @@ struct _PtyxisPreferencesWindow
   GListModel           *text_blink_modes;
   AdwSwitchRow         *use_system_font;
   AdwSwitchRow         *visual_bell;
+  GtkListBox           *custom_links_list_box;
 };
 
 G_DEFINE_FINAL_TYPE (PtyxisPreferencesWindow, ptyxis_preferences_window, ADW_TYPE_PREFERENCES_WINDOW)
@@ -359,7 +363,23 @@ static GtkWidget *
 ptyxis_preferences_window_create_profile_row_cb (gpointer item,
                                                  gpointer user_data)
 {
-  return ptyxis_profile_row_new (PTYXIS_PROFILE (item));
+  PtyxisAddButtonListItem *add_button_item = PTYXIS_ADD_BUTTON_LIST_ITEM (item);
+  GObject *wrapped_item;
+
+  wrapped_item = ptyxis_add_button_list_item_get_item (add_button_item);
+
+  if (wrapped_item != NULL)
+    {
+      return ptyxis_profile_row_new (PTYXIS_PROFILE (wrapped_item));
+    }
+  else
+    {
+      return g_object_new (ADW_TYPE_BUTTON_ROW,
+                           "title", _("Add Profile"),
+                           "start-icon-name", "list-add-symbolic",
+                           "action-name", "profile.add",
+                           NULL);
+    }
 }
 
 static gboolean
@@ -440,6 +460,29 @@ opacity_to_label (GBinding     *binding,
   return TRUE;
 }
 
+static GtkWidget *
+ptyxis_preferences_window_create_custom_link_row_cb (gpointer item,
+                                                     gpointer user_data)
+{
+  PtyxisAddButtonListItem *add_button_item = PTYXIS_ADD_BUTTON_LIST_ITEM (item);
+  GObject *wrapped_item;
+
+  g_assert (PTYXIS_IS_ADD_BUTTON_LIST_ITEM (add_button_item));
+
+  if ((wrapped_item = ptyxis_add_button_list_item_get_item (add_button_item)))
+    {
+      g_autoptr(PtyxisProfile) profile = ptyxis_application_dup_default_profile (PTYXIS_APPLICATION_DEFAULT);
+
+      return ptyxis_custom_link_row_new (profile, PTYXIS_CUSTOM_LINK (wrapped_item));
+    }
+
+  return g_object_new (ADW_TYPE_BUTTON_ROW,
+                       "title", _("Add Link"),
+                       "start-icon-name", "list-add-symbolic",
+                       "action-name", "custom-link.add",
+                       NULL);
+}
+
 static void
 ptyxis_preferences_window_notify_default_profile_cb (PtyxisPreferencesWindow *self,
                                                      GParamSpec              *pspec,
@@ -449,6 +492,8 @@ ptyxis_preferences_window_notify_default_profile_cb (PtyxisPreferencesWindow *se
   g_autoptr(GPropertyAction) palette_action = NULL;
   g_autoptr(GSettings) gsettings = NULL;
   g_autoptr(GSimpleActionGroup) group = NULL;
+  g_autoptr(GListModel) custom_links_list = NULL;
+  g_autoptr(PtyxisAddButtonListModel) custom_links_list_model = NULL;
 
   g_assert (PTYXIS_IS_PREFERENCES_WINDOW (self));
   g_assert (PTYXIS_IS_APPLICATION (app));
@@ -510,9 +555,6 @@ ptyxis_preferences_window_notify_default_profile_cb (PtyxisPreferencesWindow *se
   g_object_bind_property (profile, "scrollback-lines",
                           self->scrollback_lines, "value",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
-  g_object_bind_property (profile, "bold-is-bright",
-                          self->bold_is_bright, "active",
-                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
   g_object_bind_property (profile, "cell-height-scale",
                           self->cell_height_scale, "value",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
@@ -569,6 +611,15 @@ ptyxis_preferences_window_notify_default_profile_cb (PtyxisPreferencesWindow *se
                                 index_to_string,
                                 g_object_ref (self->preserve_directories),
                                 g_object_unref);
+
+  custom_links_list = ptyxis_profile_list_custom_links (profile);
+  custom_links_list_model = ptyxis_add_button_list_model_new (custom_links_list);
+
+  gtk_list_box_bind_model (self->custom_links_list_box,
+                           G_LIST_MODEL (custom_links_list_model),
+                           ptyxis_preferences_window_create_custom_link_row_cb,
+                           g_object_ref(self),
+                           g_object_unref);
 }
 
 static gboolean
@@ -675,6 +726,27 @@ create_palette_preview (gpointer item,
   return child;
 }
 
+static void
+ptyxis_preferences_window_add_custom_link (GtkWidget  *widget,
+                                           const char *action_name,
+                                           GVariant   *param)
+{
+  PtyxisPreferencesWindow *self = (PtyxisPreferencesWindow *)widget;
+  g_autoptr(PtyxisProfile) default_profile = NULL;
+  g_autoptr(PtyxisCustomLink) custom_link = NULL;
+  PtyxisCustomLinkEditor *editor;
+
+  g_assert (PTYXIS_IS_PREFERENCES_WINDOW (self));
+
+  default_profile = ptyxis_application_dup_default_profile (PTYXIS_APPLICATION_DEFAULT);
+  custom_link = ptyxis_custom_link_new ();
+  ptyxis_profile_add_custom_link (default_profile, custom_link);
+
+  editor = ptyxis_custom_link_editor_new (default_profile, custom_link);
+  adw_preferences_window_push_subpage (ADW_PREFERENCES_WINDOW (self),
+                                       ADW_NAVIGATION_PAGE (editor));
+}
+
 static void
 ptyxis_preferences_window_constructed (GObject *object)
 {
@@ -685,6 +757,7 @@ ptyxis_preferences_window_constructed (GObject *object)
   GSettings *gsettings = ptyxis_settings_get_settings (settings);
   AdwStyleManager *style_manager = adw_style_manager_get_default ();
   g_autoptr(GListModel) profiles = NULL;
+  g_autoptr(PtyxisAddButtonListModel) profiles_list_model = NULL;
 
   G_OBJECT_CLASS (ptyxis_preferences_window_parent_class)->constructed (object);
 
@@ -761,9 +834,10 @@ ptyxis_preferences_window_constructed (GObject *object)
                                 g_object_unref);
 
   profiles = ptyxis_application_list_profiles (app);
+  profiles_list_model = ptyxis_add_button_list_model_new (profiles);
 
   gtk_list_box_bind_model (self->profiles_list_box,
-                           profiles,
+                           G_LIST_MODEL (profiles_list_model),
                            ptyxis_preferences_window_create_profile_row_cb,
                            NULL, NULL);
 
@@ -1009,10 +1083,8 @@ ptyxis_preferences_window_class_init (PtyxisPreferencesWindowClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Ptyxis/ptyxis-preferences-window.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, add_profile_button);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, audible_bell);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, backspace_binding);
-  gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, bold_is_bright);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, cell_height_scale);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, cjk_ambiguous_width);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, cjk_ambiguous_widths);
@@ -1020,6 +1092,7 @@ ptyxis_preferences_window_class_init (PtyxisPreferencesWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, cursor_blink_modes);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, cursor_shape);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, cursor_shapes);
+  gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, custom_links_list_box);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, delete_binding);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, enable_a11y);
   gtk_widget_class_bind_template_child (widget_class, PtyxisPreferencesWindow, erase_bindings);
@@ -1095,6 +1168,10 @@ ptyxis_preferences_window_class_init (PtyxisPreferencesWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_preferences_window_show_all_cb);
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_preferences_window_spin_row_show_decimal_cb);
 
+  gtk_widget_class_install_action (widget_class,
+                                   "custom-link.add",
+                                   NULL,
+                                   ptyxis_preferences_window_add_custom_link);
   gtk_widget_class_install_action (widget_class,
                                    "profile.add",
                                    NULL,
