@@ -252,6 +252,17 @@ ptyxis_tab_update_cell_height_scale (PtyxisTab *self)
   vte_terminal_set_cell_height_scale (VTE_TERMINAL (self->terminal), cell_height_scale);
 }
 
+static void
+ptyxis_tab_update_custom_links (PtyxisTab *self)
+{
+  g_autoptr(GListModel) custom_links_list = NULL;
+
+  g_assert (PTYXIS_IS_TAB (self));
+
+  custom_links_list = ptyxis_profile_list_custom_links(self->profile);
+  ptyxis_terminal_update_custom_links_list(self->terminal, custom_links_list);
+}
+
 static gboolean
 ptyxis_tab_grab_focus (GtkWidget *widget)
 {
@@ -271,8 +282,10 @@ ptyxis_tab_wait_cb (GObject      *object,
   g_autoptr(PtyxisTab) self = user_data;
   g_autoptr(GError) error = NULL;
   PtyxisExitAction exit_action;
+  PtyxisWindow *window;
   AdwTabPage *page = NULL;
   GtkWidget *tab_view;
+  gboolean is_front = FALSE;
   int exit_code;
 
   g_assert (PTYXIS_IS_APPLICATION (app));
@@ -296,6 +309,9 @@ ptyxis_tab_wait_cb (GObject      *object,
   if (self->forced_exit)
     return;
 
+  if ((window = PTYXIS_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (self), PTYXIS_TYPE_WINDOW))))
+    is_front = self == ptyxis_window_get_active_tab (window);
+
   if (WIFSIGNALED (exit_code))
     {
       g_autofree char *title = NULL;
@@ -356,6 +372,8 @@ ptyxis_tab_wait_cb (GObject      *object,
 
     case PTYXIS_EXIT_ACTION_NONE:
       gtk_widget_set_visible (GTK_WIDGET (self->banner), TRUE);
+      if (is_front)
+        gtk_widget_child_focus (GTK_WIDGET (self->banner), GTK_DIR_TAB_FORWARD);
       break;
 
     default:
@@ -844,6 +862,13 @@ ptyxis_tab_constructed (GObject *object)
                            G_CONNECT_SWAPPED);
   ptyxis_tab_update_word_char_exceptions (self, NULL, settings);
 
+  g_signal_connect_object (G_OBJECT (self->profile),
+                           "custom-links-changed",
+                           G_CALLBACK (ptyxis_tab_update_custom_links),
+                           self,
+                           G_CONNECT_SWAPPED);
+  ptyxis_tab_update_custom_links (self);
+
   self->monitor = ptyxis_tab_monitor_new (self);
 }
 
@@ -2078,6 +2103,10 @@ ptyxis_tab_open_uri (PtyxisTab  *self,
           uri = translated = g_uri_to_string (rewritten);
         }
     }
+  else if (!g_utf8_strchr (uri, -1, ':') && g_utf8_strchr (uri, -1, '@'))
+    {
+      uri = translated = g_strconcat ("mailto:", uri, NULL);
+    }
 
   if (portal == NULL)
     portal = xdp_portal_new ();
